from html.parser import HTMLParser
from .utils import match, html_starttag_and_attrs

class Soup(HTMLParser):
    '''HTML Soup Parser

    Attributes:

    - html (str): HTML content to parse
    - tag (str, None): HTML element tag returned by find
    - attrs (dict, None): HTML element attributes returned by find
    - text (str, None): HTML element text returned by find

    Methods:

    - find: return matching HTML elements {'auto', 'all', 'first'}

    Examples:

    ```
    from gazpacho import Soup

    html = "<div><p id='foo'>bar</p><p id='foo'>baz</p><p id='zoo'>bat</p></div>"
    soup = Soup(html)

    soup.find('p', {'id': 'foo'})
    # [<p id="foo">bar</p>, <p id="foo">baz</p>]

    result = soup.find('p', {'id': 'foo'}, mode='first')
    print(result)
    # <p id="foo">bar</p>

    result = soup.find('p', {'id': 'zoo'}, mode='auto')
    print(result)
    # <p id="zoo">bat</p>

    print(result.text)
    # bat
    ```
    '''

    def __init__(self, html):
        '''Params:

        - html (str): HTML content to parse
        '''
        super().__init__()
        self.html = html
        self.tag = None
        self.attrs = None
        self.text = None

    def __dir__(self):
        return ['html', 'tag', 'attrs', 'text', 'find']

    def __repr__(self):
        return self.html

    def handle_starttag(self, tag, attrs):
        html, attrs = html_starttag_and_attrs(tag, attrs)
        matching = match(self.attrs, attrs, self.strict)
        if tag == self.tag and matching and not self.count:
            self.count += 1
            self.group += 1
            self.groups.append(Soup(''))
            self.groups[self.group - 1].html += html
            self.groups[self.group - 1].tag = tag
            self.groups[self.group - 1].attrs = attrs
            return
        if self.count:
            self.count += 1
            self.groups[self.group - 1].html += html
            return
        else:
            return

    def handle_startendtag(self, tag, attrs):
        html, attrs = html_starttag_and_attrs(tag, attrs, True)
        if self.count:
            self.groups[self.group - 1].html += html
            return
        else:
            return

    def handle_data(self, data):
        if self.count:
            if self.groups[self.group - 1].text is None:
                self.groups[self.group - 1].text = data.strip()
            self.groups[self.group - 1].html += data
            return
        else:
            return

    def handle_endtag(self, tag):
        if self.count:
            end_tag = f'</{tag}>'
            self.groups[self.group - 1].html += end_tag
            self.count -= 1
            return
        else:
            return

    def find(self, tag, attrs=None, mode='auto', strict=False):
        '''Return matching HTML elements

        Params:

        - tag (str): HTML element tag to find
        - attrs (dict, optional): HTML element attributes to match
        - mode (str, 'auto'): Element(s) to return {'auto', 'all', 'first'}
        - strict (bool, False): Require exact attribute matching

        Examples:

        ```
        html = "<div><p id='foo foo-striped'>bar</p><p id='foo'>baz</p><p id='zoo'>bat</p></div>"
        soup = Soup(html)

        soup.find('p')
        # [<p id="foo foo-striped">bar</p>, <p id="foo">baz</p>, <p id="zoo">bat</p>]

        soup.find('p', {'id': 'foo'})
        # [<p id="foo foo-striped">bar</p>, <p id="foo">baz</p>]

        result = soup.find('p', {'id': 'foo'}, mode='first')
        print(result)
        # <p id="foo">bar</p>

        soup.find('p', {'id': 'foo'}, strict=True)
        # [<p id="foo">baz</p>]
        ```
        '''
        self.tag = tag
        self.attrs = attrs
        self.strict = strict
        self.count = 0
        self.group = 0
        self.groups = []
        self.feed(self.html)
        if mode == 'all':
            return self.groups
        if mode == 'first':
            return self.groups[0]
        if mode == 'auto':
            if len(self.groups) == 1:
                return self.groups[0]
            return self.groups



def match(a, b, strict=False):
    '''Utility function to match two dictionaries

    Params:

    - a (dict): Query dictionary
    - b (dict): Dictionary to match
    - strict (bool): Require exact matching

    Examples:

    ```
    a = {'foo': 'bar'}
    b = {'foo': 'bar baz'}
    match(a, b)
    # True

    a = {'foo': 'bar'}
    b = {'foo': 'bar baz'}
    match(a, b, strict=True)
    # False

    a = {}
    b = {'foo': 'bar'}
    match(a, b)
    # True

    a = {}
    b = {}
    match(a, b)
    # True
    ```
    '''
    if not a:
        return True
    if not a and not b:
        return True
    if a and not b:
        return False
    for k, v in a.items():
        if not b.get(k):
            return False
        if strict:
            if v == b.get(k):
                continue
            else:
                return False
        if v in b.get(k):
            continue
        else:
            return False
    return True

def html_starttag_and_attrs(tag, attrs, startendtag=False):
    '''Utility functon to reconstruct starttag and attrs

    Params:

    - tag (str): HTML element tag
    - attrs (list): HTML element attributes formatted as a list of tuples
    - startendtag (bool, False): Flag to handle startend tags

    Example:

    ```
    html_starttag_and_attrs('a', [('href', 'localhost:8000')])
    # ('<a href="localhost:8000">', {'href': 'localhost:8000'})
    ```
    '''
    if attrs:
        attrs = dict(attrs)
        af = [f'{k}="{v}"' for k, v in attrs.items()]
        af = f' {" ".join(af)}'
    else:
        attrs = {}
        af = ''
    if startendtag:
        html = f'<{tag}{af} />'
    else:
        html = f'<{tag}{af}>'
    return html, attrs



from setuptools import setup

with open('README.md', encoding='utf8') as f:
    long_description = f.read()

setup(
    name='gazpacho',
    version='0.8',
    description='gazpacho is a web scraping library',
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],
    keywords=[
        'web scraping', 'web', 'scraping', 'BeautifulSoup', 'requests'
    ],
    url='https://github.com/maxhumber/gazpacho',
    author='Max Humber',
    author_email='max.humber@gmail.com',
    license='MIT',
    packages=['gazpacho'],
    python_requires='>=3.6',
    setup_requires=['setuptools>=38.6.0']
)



from urllib.parse import urlencode
from urllib.request import build_opener

def get(url, params=None, headers=None):
    '''Return the contents from a URL

    Params:

    - url (str): Target website URL
    - params (dict, optional): Param payload to add to the GET request
    - headers (dict, optional): Headers to add to the GET request

    Example:

    ```
    get('https://httpbin.org/anything', {'soup': 'gazpacho'})
    ```
    '''
    opener = build_opener()
    if params:
        url += '?' + urlencode(params)
    if headers:
        for h in headers.items():
            opener.addheaders = [h]
    if (headers and not headers.get('User-Agent')) or not headers:
        UA = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.14; rv:69.0) Gecko/20100101 Firefox/69.0'
        opener.addheaders = [('User-Agent', UA)]
    with opener.open(url) as f:
        content = f.read().decode('utf-8')
    return content
